@@ -3,7 +3,7 @@
 #
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2011 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2014 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -197,7 +197,7 @@ progenyother = {
     }
 
 def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
-    body = reason = archs = ''
+    body = reason = archs = section = priority = ''
     suite = 'unstable'
     headers = []
     pseudos = []
@@ -273,17 +273,18 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
         new_section = ui.menu('Select the new section', {
                 'admin': "", 'cli-mono': "", 'comm': "", 'database': "",
                 'debian-installer': "", 'debug': "", 'devel': "", 'doc': "",
-                'editors': "", 'electronics': "", 'embedded': "", 'fonts': "",
-                'games': "", 'gnome': "", 'gnu-r': "", 'gnustep': "",
-                'graphics': "", 'hamradio': "", 'haskell': "", 'httpd': "",
-                'interpreters': "", 'java': "", 'kde': "", 'kernel': "",
+                'editors': "", 'education': "", 'electronics': "",
+                'embedded': "", 'fonts': "", 'games': "", 'gnome': "",
+                'gnu-r': "", 'gnustep': "", 'graphics': "", 'hamradio': "",
+                'haskell': "", 'httpd': "", 'interpreters': "",
+                'introspection': "", 'java': "", 'kde': "", 'kernel': "",
                 'libdevel': "", 'libs': "", 'lisp': "", 'localization': "",
-                'mail': "", 'math': "", 'misc': "", 'net': "",
-                'news': "", 'ocaml': "", 'oldlibs': "", 'otherosfs': "",
-                'perl': "", 'php': "", 'python': "", 'ruby': "",
-                'science': "", 'shells': "", 'sound': "", 'tex': "",
-                'text': "", 'utils': "", 'vcs': "", 'video': "",
-                'web': "", 'x11': "", 'xfce': "", 'zope': "",
+                'mail': "", 'math': "", 'metapackages': "", 'misc': "",
+                'net': "", 'news': "", 'ocaml': "", 'oldlibs': "",
+                'otherosfs': "", 'perl': "", 'php': "", 'python': "",
+                'ruby': "", 'science': "", 'shells': "", 'sound': "", 'tex': "",
+                'text': "", 'utils': "", 'vcs': "", 'video': "", 'web': "",
+                'x11': "", 'xfce': "", 'zope': "",
         }, 'Choose the section: ', default=section, empty_ok=True)
         if not new_section:
             new_section = section
@@ -400,9 +401,8 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
         'britney':          "testing migration script bugs",
         'transition':       "transition tracking",
         'unblock':          "unblock requests",
-        'freeze-exception': "Freeze exceptions",
-        'opu':              "oldstable proposed updates requests",
-        'pu':               "stable proposed updates requests",
+        'squeeze-pu':       "squeeze proposed updates requests",
+        'wheezy-pu':        "wheezy proposed updates requests",
         'rm':               "Stable/Testing removal requests",
         'other' :           "None of the other options",
         }, 'Choose the request type: ', empty_ok=True)
@@ -447,14 +447,14 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
         else:
             package = info[12] or package
 
-    if tag in ('binnmu', 'unblock', 'freeze-exception', 'opu', 'pu', 'rm'):
-        # FIXME: opu/pu/rm should lookup the version elsewhere
+    if tag in ('binnmu', 'unblock', 'wheezy-pu', 'squeeze-pu', 'rm'):
+        # FIXME: pu/rm should lookup the version elsewhere
         version = info and info[0]
         if online:
-            if tag == 'pu':
-                version = checkversions.get_versions_available(package, timeout).get('stable', '')
-            elif tag == 'opu':
-                version = checkversions.get_versions_available(package, timeout).get('oldstable', '')
+            if tag == 'wheezy-pu':
+                version = checkversions.get_versions_available(package, timeout, 'wheezy').values()[0]
+            elif tag == 'squeeze-pu':
+                version = checkversions.get_versions_available(package, timeout, 'squeeze').values()[0]
         if version:
             cont = ui.select_options(
                 "Latest version seems to be %s, is this the proper one ?" % (version),
@@ -487,7 +487,11 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
                 ui.long_message('No architecture specified, skipping...')
 
     pseudos.append("User: release.debian.org@packages.debian.org")
-    pseudos.append("Usertags: %s" % (tag))
+    if tag.endswith('-pu'):
+        pseudos.append("Usertags: pu")
+        pseudos.append("Tags: %s" % (tag[:-3]))
+    else:
+        pseudos.append("Usertags: %s" % (tag))
 
     if tag == 'binnmu':
         reason  = ui.get_string("binNMU changelog entry: ")
@@ -495,20 +499,70 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
         body    = "nmu %s_%s . %s . -m \"%s\"\n" % (package, version, archs or "ALL", reason)
     elif tag == 'transition':
         subject = 'transition: %s' % (package)
-        body    = '(please explain about the transition: impacted packages, reason, ...)\n'
+        body    = '(please explain about the transition: impacted packages, reason, ...\n' \
+                  ' for more info see: https://wiki.debian.org/Teams/ReleaseTeam/Transitions)\n'
+        affected = '<Fill out>'
+        good = '<Fill out>'
+        bad = '<Fill out>'
+
+        ui.long_message('To assist the release team, please fill in the following information. '
+                        'You will be asked to provide package names of the library package(s) '
+                        'that are the source of the transition.  If more than one library is '
+                        'changing the name, please use a space separated list.  Alternatively '
+                        'you can use a regex by enclosing the regex in slashes ("/").  Please '
+                        'ensure that the "old" regex does not match the "new" packages.  '
+                        'Example: old="/libapt-pkg4.10|libapt-inst1.2/ libept1" '
+                        'new="/libapt-pkg4.12|libapt-inst1.5|libept1.4.12/". For futher '
+                        'reference, please refer to http://ben.debian.net/ .')
+
+        prompt = 'Please enter old binary package name of the library (or a regex matching it):'
+        tfrom = ui.get_string(prompt)
+        if tfrom:
+            prompt = 'Please enter new binary package name of the library (or a regex matching it):'
+            tto = ui.get_string(prompt)
+        else:
+            tto = None
+        if tfrom and tto:
+            # Compute a ben file from this.
+
+            # (quote if x does not start with a "/")
+            quote=lambda x: (x[0] == '/' and x) or '"%s"' % x
+
+            listbad = [quote(x) for x in tfrom.strip().split()]
+            listgood = [quote(x) for x in tto.strip().split()]
+
+            j = " | .depends ~ ".join
+            affected = ".depends ~ " + j(listbad + listgood)
+            good = ".depends ~ " + j(listgood)
+            bad = ".depends ~ " + j(listbad)
+
+
+        body += textwrap.dedent(u"""\
+
+               Ben file:
+
+               title = "%s";
+               is_affected = %s;
+               is_good = %s;
+               is_bad = %s;
+
+               """ % (package, affected, good, bad))
+
     elif tag == 'britney':
         subject = subject_britney
         body = ''
-    elif tag == 'unblock' or tag == 'freeze-exception':
+    elif tag == 'unblock':
         subject = 'unblock: %s/%s' % (package, version)
         body    = textwrap.dedent(u"""\
                 Please unblock package %s
 
                 (explain the reason for the unblock here)
 
+                (include/attach the debdiff against the package in testing)
+
                 unblock %s/%s
                 """ % (package, package, version))
-    elif tag == 'pu' or tag == 'opu':
+    elif tag.endswith('-pu'):
         subject = '%s: package %s/%s' % (tag, package, version)
         body    = '(please explain the reason for this update here)\n'
     elif tag == 'rm':
@@ -527,6 +581,15 @@ itp_template = textwrap.dedent(u"""\
       Description     : %(short_desc)s
 
     (Include the long description here.)
+
+    Please also include as much relevant information as possible.
+    For example, consider answering the following questions:
+     - why is this package useful/relevant? is it a dependency for
+       another package? do you use it? if there are other packages
+       providing similar functionality, how does it compare?
+     - how do you plan to maintain it? inside a packaging team
+       (check list at https://wiki.debian.org/Teams)? are you
+       looking for co-maintainers? do you need a sponsor?
 """)
 
 
@@ -564,6 +627,9 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
         else:
             prompt = 'Please enter the package name: '
         package = ui.get_string(prompt)
+        if not utils.check_package_name(package):
+            ui.long_message('Invalid package name')
+            package = ""
 
     ui.log_message('Checking status database...\n')
     info = utils.get_package_status(package, avail=True)
@@ -649,14 +715,18 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
 
 def dpkg_infofunc():
     debarch = utils.get_arch()
+    utsmachine = os.uname()[4]
+    multiarch = utils.get_multiarch()
     if debarch:
-        utsmachine = os.uname()[4]
         if utsmachine == debarch:
-            debinfo = u'Architecture: %s\n\n' % debarch
+            debinfo = u'Architecture: %s\n' % debarch
         else:
-            debinfo = u'Architecture: %s (%s)\n\n' % (debarch, utsmachine)
+            debinfo = u'Architecture: %s (%s)\n' % (debarch, utsmachine)
     else:
-        debinfo = u'Architecture: ? (%s)\n\n' % utsmachine
+        debinfo = u'Architecture: ? (%s)\n' % utsmachine
+    if multiarch:
+        debinfo += u'Foreign Architectures: %s\n' % multiarch
+    debinfo += '\n'
     return debinfo
 
 def debian_infofunc():
